require 'rainbow'
require 'logger'

namespace :import do
  desc 'Create educational stage relations based on its metadata'
  task :educational_stage, [:log] => [:environment] do |_t, _args|
    logger = Logger.new(STDOUT)
    educational_relations = {}
    tags = []
    high_stages = ['Educação Superior', 'Ensino Terciário', 'Ensino Superior']

    EducationalStage.all.each do |educational|
      logger.info("Creating relations of #{educational.name}")

      if educational.name == 'Ensino Superior'
        high_stages.select { |stage| tag_exists?(stage) }.each do |stage|
          tags << Tag.find_by_name(stage)
        end
      elsif educational.name == 'Outros'
        tags = Tag.where.not(name: EducationalStage.all.select { |e| e.name != 'Outros' }.map(&:name) + high_stages)
      elsif tag_exists?(educational.name)
        tags << Tag.find_by_name(educational.name)
      end

      tags.each do |tag|
        tag.taggings.each do |tagging|
          next if stage_relation_exists?(educational, tagging.taggable_id, tagging.taggable_type)
          educational_relation = StageRelation.new(educational_stage: educational, stageable_id: tagging.taggable_id, stageable_type: tagging.taggable_type)
          educational_relations[[educational, tagging.taggable_id, tagging.taggable_type]] = educational_relation
        end
      end

      tags = []
    end

    StageRelation.import educational_relations.values
  end

  private

  def tag_exists?(name)
    Tag.where(name: name).exists?
  end

  def stage_relation_exists?(educational, id, type)
    StageRelation.where(educational_stage: educational, stageable_id: id, stageable_type: type).exists?
  end
end
